import React from 'react'
import d3 from './bin'

import {
  getTooltipOption,
  getLegendOption,
  getSeriesOption,
} from '../generator'

const generator = (option: Ioptions) => {
  const { chartStyles, data, cols } = option
  const { legend, label, spec } = chartStyles

  const legendOption = {
    type: 'pie',
    legend,
    data,
    cols,
  }

  const seriesOption = {
    type: 'pie',
    data,
    label,
    cols,
    spec,
  }

  return {
    tooltip: getTooltipOption('pie'),
    legend: getLegendOption(legendOption),
    series: getSeriesOption(seriesOption),
  }
}

let chart: any
const initChart = (type: string, id: HTMLDivElement, option: Ioptions) => {
  if (type === 'init') {
    chart = d3.init(id)
  }
  chart.setOption(generator(option))
}

interface IProps {
  options: Ioptions
}

class Chart extends React.Component<IProps, any> {
  div: HTMLDivElement

  componentDidMount() {
    const { options } = this.props
    initChart('init', this.div, options)
  }

  componentDidUpdate() {
    const { options } = this.props
    initChart('update', this.div, options)
  }

  render() {
    const { width, height } = this.props.options
    return(
      <div
        style={{ width, height, position: 'absolute' }}
        ref={(node) => this.div = node}
      />
    )
  }
}

export default Chart
