import * as d3Select from 'd3-selection'
import * as d3Aixs from 'd3-axis'
import * as d3Scale from 'd3-scale'
import * as d3Transition from 'd3-transition'
import * as d3Ease from 'd3-ease'

import { getElem } from './util/getElem'
import XAxis from './component/XAxis'
import YAxis from './component/YAxis'
import Series from './component/Series/index'
import Tooltip from './component/Tooltip'
import Legend from './component/Legend'
import { getMaxNum } from './util/getSeries'

const d3 = Object.assign(
  {},
  d3Select,
  d3Aixs,
  d3Scale,
  d3Transition,
  d3Ease,
)

class Chart {
  xAxis: any
  yAxis: any
  tooltip: any
  series: any
  legend: any
  common: any
  HTMLDom: HTMLDivElement
  elem: HTMLOrSVGElement
  options: any

  constructor(HTMLDom: any) {
    this.HTMLDom = HTMLDom
  }

  setOption(options: any) {
    this.init(this.HTMLDom, options)
  }

  init(el: HTMLDivElement, options: any) {

    const elem = d3.select(el)
    const svg = getElem(elem, 'svg', 'yo-chart', '100%', '100%')

    const width = elem.node().offsetWidth
    const height = elem.node().offsetHeight

    const chartWidth = width * 0.8
    const chartHeight = height * 0.8
    const paddingLeft = width * 0.1
    const paddingTop = height * 0.1

    this.common = {
      width,
      height,
      chartWidth,
      chartHeight,
      paddingLeft,
      paddingTop,
      svg,
    }

    this.options = options
    this.elem = elem

    const { max, ticks } = this.getMaxTicks()

    const fn: { [key: string]: () => void } = {
      xAxis: () => {
        const xAxisOption = {
          ...this.common,
          xAxis: options.xAxis,
        }
        if (!this.xAxis) {
          this.xAxis = new XAxis()
        }
        this.xAxis.init(xAxisOption)
      },
      yAxis: () => {
        const yAxisOption = {
          ...this.common,
          yAxis: options.yAxis,
          max,
          ticks,
        }
        if (!this.yAxis) {
          this.yAxis = new YAxis()
        }
        this.yAxis.init(yAxisOption)
      },
      tooltip: () => {
        const tooltipOption = {
          elem,
          width,
          height,
          series: options.series,
          xAxis: options.xAxis,
        }
        if (!this.tooltip) {
          this.tooltip = new Tooltip()
        }
        this.tooltip.init(tooltipOption)
      },
      legend: () => {
        const legendOption = {
          elem,
          legend: options.legend,
        }
        if (!this.legend) {
          this.legend = new Legend()
        }
        this.legend.initDIV(legendOption, this.legendHandler.bind(this))
      },
      series: () => {
        const seriesOption = {
          ...this.common,
          series: options.series,
          max,
        }
        if (!this.series) {
          this.series = new Series()
        }
        this.series.init(seriesOption, this.tooltip)
      },
    }

    for (const x in options) {
      fn[x]()
    }
  }

  legendHandler(index: any) {
    const { max, ticks } = this.getMaxTicks(index)

    const { width, height } = this.common

    const fn: any = {
      tooltip: () => {
        const tooltipOption = {
          elem: this.elem,
          width,
          height,
          series: this.options.series,
          xAxis: this.options.xAxis,
        }
        this.tooltip.init(tooltipOption, index)
      },
      yAxis: () => {
        const yAxisOption = {
          ...this.common,
          yAxis: this.options.yAxis,
          max,
          ticks,
        }
        this.yAxis.init(yAxisOption)
      },
      series: () => {
        const seriesOption = {
          ...this.common,
          series: this.options.series,
          max,
        }
        this.series.init(seriesOption, this.tooltip, index)
      },
    }

    for (const x in this.options) {
      fn[x] && fn[x]()
    }
  }

  // 取数的最大值作为 y 轴最大值, 以及刻度数
  getMaxTicks(index?: any) {
    const newSeries = this.options.series.filter((item: any) => !index && true || index[item.name])
    return getMaxNum(newSeries)
  }
}

export default Chart
