import { getElem } from '../util/getElem'
import { color } from '../config'

interface Ilegend {
  legend: any
  elem: any
}

class Legend {
  status: any
  textStyleColor: string
  grey: string

  constructor() {
    this.status = {}
    this.textStyleColor = null
    this.grey = '#c5c5c5'
  }

  // 生成 div 形式
  initDIV(legendOption: Ilegend, legendHandler: any) {
    const {
      legend,
      elem,
    } = legendOption

    const {
      data,
      show,
      textStyle,
      top,
      left,
      bottom,
      right,
      orient,
    } = legend
    const legendDiv = getElem(elem, 'div', 'yo-legend')

    // legend 每次更新时全选中状态
    this.status = legend.selected

    legendDiv
      .style('position', 'absolute')
      .style('visibility', show && 'visible' || 'hidden')
      .style('display', 'flex')
      .style('flex-direction', orient === 'horizontal' ? 'row' : 'column')
      .style('left', left)
      .style('right', right)
      .style('top', top)
      .style('bottom', bottom)
      .style('padding', '5px')

    const options = {
      legendDiv,
      data,
      textStyle,
      legendHandler,
    }
    this.rectLegend(options)
  }

  /**
   * 生成矩形块 legend
   */
  rectLegend(options: any) {
    const {
      legendDiv,
      data,
      textStyle,
      legendHandler,
    } = options

    const update = legendDiv.selectAll('.yo-legend-li').data(data)
    const enter = update.enter()
    const exit = update.exit()

    const self = this
    this.textStyleColor = textStyle.color

    // 点击 div
    const li = enter.append('div')
      .attr('class', 'yo-legend-li')
      .attr('style', 'margin: 3px 16px 3px 0; display: flex; align-items: center; cursor: pointer')
      .on('click', function(e: any, index: number) {
        self.status[e] = !self.status[e]
        if (self.status[e]) {
          this.getElementsByTagName('div')[0].style.background = color[index]
          this.getElementsByTagName('span')[0].style.color = self.textStyleColor
        } else {
          this.getElementsByTagName('div')[0].style.background = self.grey
          this.getElementsByTagName('span')[0].style.color = self.grey
        }
        legendHandler(self.status)
      })

    // 矩形块
    li.append('div')
      .attr('class', 'yo-legend-li-rect')
      .attr('style', (d: any, i: number) => `width: 24px;
        height: 14px; background: ${this.status[d] && color[i] || this.grey};
        margin-right: 5px; border-radius: 3px`)

    // 文字
    li.append('span')
      .attr('class', 'yo-legend-li-text')
      .attr('style', `font-size: ${textStyle.fontSize}px;
        color: ${textStyle.color};
        font-family: ${textStyle.fontFamily}`)
      .text((d: any) => d)

    update.select('.yo-legend-li-rect')
      .style('background', (d: any, i: number) => this.status[d] && color[i] || this.grey)
    update.select('.yo-legend-li-text')
      .attr('style', `font-size: ${textStyle.fontSize}px;
        color: ${textStyle.color};
        font-family: ${textStyle.fontFamily}`)
      .text((d: any) => d)

    exit.remove()
  }

  /**
   * 生成圆形块 legend
   */
  circleLegend(options: any) {
    const {
      legendDiv,
      data,
      textStyle,
      legendHandler,
    } = options

    const update = legendDiv.selectAll('.yo-legend-li').data(data)
    const enter = update.enter()
    const exit = update.exit()

    const self = this
    this.textStyleColor = textStyle.color

    // 点击 div
    const li = enter.append('div')
      .attr('class', 'yo-legend-li')
      .attr('style', 'margin: 3px 16px 3px 0; display: flex; align-items: center; cursor: pointer')
      .on('click', function(e: any, index: number) {
        self.status[e] = !self.status[e]
        if (self.status[e]) {
          this.getElementsByTagName('div')[0].style.background = color[index]
          this.getElementsByTagName('span')[0].style.color = self.textStyleColor
        } else {
          this.getElementsByTagName('div')[0].style.background = this.grey
          this.getElementsByTagName('span')[0].style.color = this.grey
        }
        legendHandler(self.status)
      })

    // 矩形块
    li.append('div')
      .attr('class', 'yo-legend-li-rect')
      .attr('style', (d: any, i: number) => `width: 14px;
        height: 14px; background: ${this.status[d] && color[i] || this.grey};
        margin-right: 5px; border-radius: 50%`)

    // 文字
    li.append('span')
      .attr('class', 'yo-legend-li-text')
      .attr('style', `font-size: ${textStyle.fontSize}px;
        color: ${textStyle.color};
        font-family: ${textStyle.fontFamily}`)
      .text((d: any) => d)

    update.select('.yo-legend-li-rect')
      .style('background', (d: any, i: number) => this.status[d] && color[i] || this.grey)
    update.select('.yo-legend-li-text')
      .attr('style', `font-size: ${textStyle.fontSize}px;
        color: ${textStyle.color};
        font-family: ${textStyle.fontFamily}`)
      .text((d: any) => d)

    exit.remove()
  }
}

export default Legend
