import Scale from '../../../coord/Scale'
import render from './render'
import { color } from '../../../config'
import trigger from '../../../shape/trigger'

interface IbarOption {
  chartWidth: number
  chartHeight: number
  paddingLeft: number
  paddingTop: number
  svg: any
  bardata: any
  max: number
}

/**
 * generate bar
 * @param {IbarOption} barOption
 * @return {null}
 */
class Bar {
  duration: number

  constructor() {
    this.duration = 500
  }

  init(barOption: IbarOption, tooltip: any, showList?: any) {
    const {
      chartWidth,
      chartHeight,
      paddingLeft,
      paddingTop,
      svg,
      bardata,
      max,
    } = barOption

    // 比例尺
    const scaleOption = {
      domain: [0, max],
      range: [0, chartHeight],
    }
    const scale = Scale(scaleOption)

    const per = bardata[0].data.length // 多少份

    const showData = showList && bardata.filter((item: any) => {
      return showList[item.name]
    }) || bardata

    // 矩形 rect
    const len = showData.length // 多少组数据, 即一份矩形条数
    const rectBoxWidth = chartWidth / per // 每一份矩形宽度
    const rectWidth = len && rectBoxWidth / 1.6 / len // 矩形宽度
    const gap = rectBoxWidth / 14 // 矩形之间的间隙
    const rectPaddingLeft = (rectBoxWidth - ((len - 1) * gap + rectWidth * len)) / 2 // 第一组柱状图距左侧的距离

    let key = 0
    bardata.map((item: any, index: number) => {
      let rectPaddingLefts = 0
      if (showList) {
        if (showList[item.name]) {
          rectPaddingLefts = rectPaddingLeft + (rectWidth + gap) * key
          key += 1
        } else {
          rectPaddingLefts = rectPaddingLeft + (rectWidth + gap) * key
        }
      } else {
        rectPaddingLefts = rectPaddingLeft + (rectWidth + gap) * index
      }

      const data = (!showList || showList[item.name]) && item.data || []

      const chartOption = {
        chartHeight,
        paddingLeft,
        paddingTop,
        svg,
        rectBoxWidth,
        rectWidth,
        rectGap: gap,
        rectPaddingLeft: rectPaddingLefts, // 每组柱状图距左侧的距离
        scale,
        data,
        label: item.label,
        gname: `yo-barchart-${index}`,
        rectname: `yo-rect-${index}`,
        color: color[index],
        duration: this.duration,
      }
      render(chartOption)
    })

    // 生成 trigger
    const triggerOption = {
      chartHeight,
      paddingLeft,
      paddingTop,
      svg,
      data: bardata,
      rectBoxWidth,
      tooltip,
    }
    trigger(triggerOption)
  }
}

export default Bar
