import { getElem } from '../../../util/getElem'
import generateLabel from '../../../shape/label'

interface IchartOption {
  chartHeight: number
  paddingLeft: number
  paddingTop: number
  svg: HTMLDocument
  data: any
  label: any
  rectBoxWidth: number
  rectWidth: number
  scale: any
  gname: string
  rectname: string
  rectPaddingLeft: number
  color: string
  duration: number
}

/**
 * generate every bar
 * @param {IchartOption} chartOption
 * @return {null}
 */
const chart = (chartOption: IchartOption) => {
  const {
    chartHeight,
    paddingLeft,
    paddingTop,
    svg,
    data,
    label,
    rectBoxWidth,
    rectWidth,
    scale,
    gname,
    rectname,
    rectPaddingLeft,
    color,
    duration,
  } = chartOption

  // 图 gp
  const gp = getElem(svg, 'g', gname)
  gp.attr('transform', `translate(${paddingLeft}, ${paddingTop})`)

  const update = gp.selectAll(`.${rectname}`).data(data)
  const enter = update.enter()
  const exit = update.exit()

  enter.append('rect')
    .attr('class', rectname)
    .attr('width', rectWidth)
    .attr('height', 0)
    .attr('y', chartHeight)
    .attr('style', `fill: ${color}; cursor: pointer`)
    .attr('transform', (d: any, i: any) => `translate(${rectBoxWidth * i + rectPaddingLeft}, 0)`)
    .transition()
    .duration(duration)
    .attr('height', (d: any) => scale(d))
    .attr('y', (d: any) => chartHeight - scale(d))

  update
    .transition()
    .duration(duration)
    .attr('width', rectWidth)
    .attr('height', (d: any) => scale(d))
    .attr('y', (d: any) => chartHeight - scale(d))
    .attr('transform', (d: any, i: any) => `translate(${rectBoxWidth * i + rectPaddingLeft}, 0)`)
    .style('fill', color)

  exit.remove()

  // label 位置
  const position: { [key: string]: any } = {
    inside: {
      x: (i: any) => rectBoxWidth * i + rectPaddingLeft + rectWidth / 2,
      y: (d: any) => chartHeight - scale(d) / 2,
    },
    insideTop: {
      x: (i: any) => rectBoxWidth * i + rectPaddingLeft + rectWidth / 2,
      y: (d: any) => chartHeight - scale(d) + 20,
    },
    top: {
      x: (i: any) => rectBoxWidth * i + rectPaddingLeft + rectWidth / 2,
      y: (d: any) => chartHeight - scale(d) - 6,
    },
    bottom: {
      x: (i: any) => rectBoxWidth * i + rectPaddingLeft + rectWidth / 2,
      y: (d: any) => chartHeight + 20,
    },
    left: {
      x: (i: any) => rectBoxWidth * i + rectPaddingLeft - rectWidth / 2,
      y: (d: any) => chartHeight - scale(d) / 2,
    },
    right: {
      x: (i: any) => rectBoxWidth * i + rectPaddingLeft + rectWidth * 1.5,
      y: (d: any) => chartHeight - scale(d) / 2,
    },
  }

  // 生成 label 文字
  const labelOption = {
    gp,
    data,
    label,
    duration,
    exitDuration: 0,
    name: 'label-text',
    y1: chartHeight,
    x2: position[label.position].x,
    y2: position[label.position].y,
  }
  generateLabel(labelOption)

}

export default chart
