import * as d3Select from 'd3-selection'
import * as d3Ease from 'd3-ease'
import * as d3Array from 'd3-array'
import Bar from './bar/index'
import Line from './line/index'
import Scatter from './scatter/index'
import Pie from './pie/index'

const d3 = Object.assign(
  {},
  d3Select,
  d3Ease,
  d3Array,
)

interface Iseries {
  chartWidth: number
  chartHeight: number
  paddingLeft: number
  paddingTop: number
  svg: HTMLDocument
  series: any[]
  max: number
}

class Series {
  series: any
  bar: any
  line: any
  scatter: any
  pie: any

  constructor() {
    this.series = null
    this.bar = new Bar()
    this.line = new Line()
    this.scatter = new Scatter()
    this.pie = new Pie()
  }

  init(seriesOption: Iseries, tooltip: any, showList?: any) {
    const {
      series,
      max,
      ...common
    } = seriesOption

    this.series = series

    // 取类型为 bar 的数据
    const bardata = series.filter((item) => item.type === 'bar' && item)

    if (bardata.length > 0) {
      const barOption = {
        ...common,
        bardata,
        max,
      }
      this.bar.init(barOption, tooltip, showList)
    }

    const linedata = series.filter((item) => item.type === 'line' && item)

    if (linedata.length > 0) {
      const lineOption = {
        ...common,
        linedata,
        max,
      }
      this.line.init(lineOption, tooltip, showList)
    }

    const scatterdata = series.filter((item) => item.type === 'scatter' && item)

    if (scatterdata.length > 0) {
      const scatterOption = {
        ...common,
        scatterdata,
        max,
      }
      this.scatter.init(scatterOption, tooltip, showList)
    }

    const piedata = series.filter((item) => item.type === 'pie' && item)

    if (piedata.length > 0) {
      const pieOption = {
        ...common,
        piedata,
        max,
      }
      this.pie.init(pieOption, tooltip, showList)
    }
  }
}

export default Series
