import Scale from '../../../coord/Scale'
import render from './render'
import { color } from '../../../config'
import trigger from '../../../shape/trigger'
import clipPath from '../../../shape/clipPath'

interface IlineOption {
  chartWidth: number
  chartHeight: number
  paddingLeft: number
  paddingTop: number
  svg: any
  linedata: any
  max: number
}

/**
 * generator
 * @param {IlineOption} lineOption
 * @return {null}
 */
class Line {
  duration: number

  constructor() {
    this.duration = 500
  }

  init(lineOption: IlineOption, tooltip: any, showList?: any) {
    const {
      chartWidth,
      chartHeight,
      paddingLeft,
      paddingTop,
      svg,
      linedata,
      max,
    } = lineOption

    // 比例尺
    const scaleOption = {
      domain: [0, max],
      range: [0, chartHeight],
    }
    const scale = Scale(scaleOption)

    const per = linedata[0].data.length // 多少份

    // 矩形 rect
    const rectBoxWidth = chartWidth / per // 每一份矩形宽度

    linedata.map((item: any, index: number) => {

      const data = (!showList || showList[item.name]) && item.data || []

      const chartOption = {
        chartHeight,
        paddingLeft,
        paddingTop,
        svg,
        rectBoxWidth,
        scale,
        data,
        label: item.label,
        gname: `yo-linechart-${index}`,
        linename: `yo-line-${index}`,
        color: color[index],
        duration: this.duration,
      }
      render(chartOption)
    })

    // 生成 trigger
    const triggerOption = {
      chartHeight,
      paddingLeft,
      paddingTop,
      svg,
      data: linedata,
      rectBoxWidth,
      tooltip,
    }
    trigger(triggerOption)

    const clipPathOption = {
      svg,
      chartWidth,
      chartHeight,
    }
    clipPath(clipPathOption)
  }
}

export default Line
