import * as d3Select from 'd3-selection'
import * as d3Shape from 'd3-shape'
import { getElem } from '../../../util/getElem'
import generateLabel from '../../../shape/label'

const d3 = Object.assign(
  {},
  d3Select,
  d3Shape,
)

interface IchartOption {
  chartHeight: number
  paddingLeft: number
  paddingTop: number
  svg: HTMLDocument
  data: any
  label: any
  rectBoxWidth: number
  scale: any
  gname: string
  linename: string
  color: string
  duration: number
}

/**
 * generate and animation
 * @param {IchartOption} chartOption
 * @return {null}
 */
const chart = (chartOption: IchartOption) => {
  const {
    chartHeight,
    paddingLeft,
    paddingTop,
    svg,
    data,
    label,
    rectBoxWidth,
    scale,
    gname,
    linename,
    color,
    duration,
  } = chartOption

  // 图 gp
  const gp = getElem(svg, 'g', gname)
  gp.attr('transform', `translate(${paddingLeft}, ${paddingTop})`)
    .attr('clip-path', 'url(#clipPath)')

  const update = gp.selectAll(`.${linename}`).data(data[0] && [data[0]] || [])
  const enter = update.enter()
  const exit = update.exit()

  const line = d3.line()
    .x((d, i) => rectBoxWidth * i)
    .y((d) => chartHeight - scale(d))

  enter.append('path')
    .attr('class', linename)
    .attr('d', () => line(data))
    .attr('stroke', color)
    .attr('style', `fill: none; cursor: pointer`)
    .attr('transform', `translate(${rectBoxWidth / 2}, 0)`)
    .style('opacity', 0)
    .transition()
    .duration(duration)
    .style('opacity', 1)

  update
    .transition()
    .duration(duration)
    .attr('d', () => line(data))
    .attr('stroke', color)
    .attr('transform', `translate(${rectBoxWidth / 2}, 0)`)

  exit
    .transition()
    .duration(200)
    .style('opacity', 0)
    .remove()

  // label 位置
  const position: { [key: string]: any } = {
    inside: {
      x: (i: any) => rectBoxWidth * i + rectBoxWidth / 2,
      y: (d: any) => chartHeight - scale(d),
    },
    insideTop: {
      x: (i: any) => rectBoxWidth * i + rectBoxWidth / 2,
      y: (d: any) => chartHeight - scale(d) - 10,
    },
    top: {
      x: (i: any) => rectBoxWidth * i + rectBoxWidth / 2,
      y: (d: any) => chartHeight - scale(d) - 10,
    },
    bottom: {
      x: (i: any) => rectBoxWidth * i + rectBoxWidth / 2,
      y: (d: any) => chartHeight - scale(d) + 20,
    },
    left: {
      x: (i: any) => rectBoxWidth * i + rectBoxWidth / 2 - 12,
      y: (d: any) => chartHeight - scale(d) + 4,
    },
    right: {
      x: (i: any) => rectBoxWidth * i + rectBoxWidth / 2 + 12,
      y: (d: any) => chartHeight - scale(d) + 4,
    },
  }

  // 生成 label 文字
  const labelOption = {
    gp,
    data,
    label,
    duration,
    name: 'label-text',
    y1: position[label.position].y,
    x2: position[label.position].x,
    y2: position[label.position].y,
  }
  generateLabel(labelOption)

}

export default chart
