import Scale from '../../../coord/Scale'
import render from './render'
import { color } from '../../../config'
import trigger from '../../../shape/trigger'

interface IscatterOption {
  chartWidth: number
  chartHeight: number
  paddingLeft: number
  paddingTop: number
  svg: any
  scatterdata: any
  max: number
}

/**
 * generator
 * @param {IscatterOption} scatterOption
 * @return {null}
 */
class Scatter {
  duration: number

  constructor() {
    this.duration = 500
  }

  init(scatterOption: IscatterOption, tooltip: any, showList?: any) {
    const {
      chartWidth,
      chartHeight,
      paddingLeft,
      paddingTop,
      svg,
      scatterdata,
      max,
    } = scatterOption

    // 比例尺
    const scaleOption = {
      domain: [0, max],
      range: [0, chartHeight],
    }
    const scale = Scale(scaleOption)

    const per = scatterdata[0].data.length // 多少份

    // 矩形 rect
    const rectBoxWidth = chartWidth / per // 每一份矩形宽度

    scatterdata.map((item: any, index: number) => {

      const data = (!showList || showList[item.name]) && item.data || []

      const chartOption = {
        chartHeight,
        paddingLeft,
        paddingTop,
        svg,
        rectBoxWidth,
        scale,
        data,
        label: item.label,
        gname: `yo-scatterchart-${index}`,
        scattername: `yo-scatter-${index}`,
        color: color[index],
        duration: this.duration,
      }
      render(chartOption)
    })

    // 生成 trigger
    const triggerOption = {
      chartHeight,
      paddingLeft,
      paddingTop,
      svg,
      data: scatterdata,
      rectBoxWidth,
      tooltip,
    }
    trigger(triggerOption)
  }
}

export default Scatter
