import { getElem } from '../../../util/getElem'
import generateLabel from '../../../shape/label'

interface IchartOption {
  chartHeight: number
  paddingLeft: number
  paddingTop: number
  svg: HTMLDocument
  data: any
  label: any
  rectBoxWidth: number
  scale: any
  gname: string
  scattername: string
  color: string
  duration: number
}

/**
 * generate and animation
 * @param {IchartOption} chartOption
 * @return {null}
 */
const chart = (chartOption: IchartOption) => {
  const {
    chartHeight,
    paddingLeft,
    paddingTop,
    svg,
    data,
    label,
    rectBoxWidth,
    scale,
    gname,
    scattername,
    color,
    duration,
  } = chartOption

  // 图 gp
  const gp = getElem(svg, 'g', gname)
  gp.attr('transform', `translate(${paddingLeft}, ${paddingTop})`)

  const update = gp.selectAll(`.${scattername}`).data(data)
  const enter = update.enter()
  const exit = update.exit()

  enter.append('circle')
    .attr('class', scattername)
    .attr('cx', (d: any, i: number) => rectBoxWidth * i)
    .attr('cy', (d: any) => chartHeight - scale(d))
    .attr('r', 0)
    .attr('style', `fill: ${color}; cursor: pointer`)
    .attr('transform', `translate(${rectBoxWidth / 2}, 0)`)
    .transition()
    .duration(500)
    .attr('r', 5)

  update
    .transition()
    .duration(duration)
    .attr('cx', (d: any, i: number) => rectBoxWidth * i)
    .attr('cy', (d: any) => chartHeight - scale(d))
    .attr('r', 5)
    .attr('style', `fill: ${color}; cursor: pointer`)
    .attr('transform', `translate(${rectBoxWidth / 2}, 0)`)

  exit
    .transition()
    .duration(duration)
    .attr('r', 0)
    .remove()

  // label 位置
  const position: { [key: string]: any } = {
    inside: {
      x: (i: any) => rectBoxWidth * i + rectBoxWidth / 2,
      y: (d: any) => chartHeight - scale(d),
    },
    insideTop: {
      x: (i: any) => rectBoxWidth * i + rectBoxWidth / 2,
      y: (d: any) => chartHeight - scale(d) - 10,
    },
    top: {
      x: (i: any) => rectBoxWidth * i + rectBoxWidth / 2,
      y: (d: any) => chartHeight - scale(d) - 10,
    },
    bottom: {
      x: (i: any) => rectBoxWidth * i + rectBoxWidth / 2,
      y: (d: any) => chartHeight - scale(d) + 20,
    },
    left: {
      x: (i: any) => rectBoxWidth * i + rectBoxWidth / 2 - 16,
      y: (d: any) => chartHeight - scale(d) + 4,
    },
    right: {
      x: (i: any) => rectBoxWidth * i + rectBoxWidth / 2 + 16,
      y: (d: any) => chartHeight - scale(d) + 4,
    },
  }

  // 生成 label 文字
  const labelOption = {
    gp,
    data,
    label,
    duration,
    name: 'label-text',
    y1: position[label.position].y,
    x2: position[label.position].x,
    y2: position[label.position].y,
  }
  generateLabel(labelOption)

}

export default chart
