import { getElem } from '../util/getElem'
import { lineStyleType } from '../config'

interface IsplitLine {
  splitLine: any
  chartWidth: number
  chartHeight: number
  paddingLeft: number
  paddingTop: number
  svg: HTMLDocument
  len?: number
  ticks?: number
}

class SplitLine {

  // 生成 xAxis splitline (垂直线)
  initX(splitLineOption: IsplitLine) {
    const {
      splitLine,
      chartWidth,
      chartHeight,
      paddingLeft,
      paddingTop,
      svg,
      len,
    } = splitLineOption
    const gxs = getElem(svg, 'g', 'yo-split-vertical')

    const gxsUpdate = gxs.selectAll('.yo-split-line').data(new Int8Array(len))
    const gxsEnter = gxsUpdate.enter()
    const gxsExit = gxsUpdate.exit()

    gxsEnter.append('line')
      .attr('class', `yo-split-line`)
      .attr('x1', (d: any, i: number) => chartWidth / len * (i + 1) + paddingLeft + .5)
      .attr('y1', paddingTop)
      .attr('x2', (d: any, i: number) => chartWidth / len * (i + 1) + paddingLeft + .5)
      .attr('y2', paddingTop + chartHeight)
      .attr('stroke-dasharray', lineStyleType[splitLine.lineStyle.type])
      .style('stroke', splitLine.lineStyle.color)
      .style('stroke-width', splitLine.lineStyle.width)
      .style('visibility', splitLine.show ? 'visible' : 'hidden')

    gxsUpdate
      .attr('x1', (d: any, i: number) => chartWidth / len * (i + 1) + paddingLeft + .5)
      .attr('y1', paddingTop)
      .attr('x2', (d: any, i: number) => chartWidth / len * (i + 1) + paddingLeft + .5)
      .attr('y2', paddingTop + chartHeight)
      .attr('stroke-dasharray', lineStyleType[splitLine.lineStyle.type])
      .style('stroke', splitLine.lineStyle.color)
      .style('stroke-width', splitLine.lineStyle.width)
      .style('visibility', splitLine.show ? 'visible' : 'hidden')

    gxsExit
      .style('stroke', 'transparent')
      .remove()
  }

  // 生成 yAxis splitline (水平线)
  initY(splitLineOption: IsplitLine) {
    const {
      splitLine,
      chartWidth,
      chartHeight,
      paddingLeft,
      paddingTop,
      svg,
      ticks,
    } = splitLineOption

    const gys = getElem(svg, 'g', 'yo-split-horizontal')

    const gysUpdate = gys.selectAll('.yo-split-line').data(new Int8Array(ticks))
    const gysEnter = gysUpdate.enter()
    const gysExit = gysUpdate.exit()

    gysEnter.append('line')
      .attr('class', `yo-split-line`)
      .attr('x1', paddingLeft)
      .attr('y1', (d: any, i: number) => paddingTop + chartHeight / ticks * i + .5)
      .attr('x2', paddingLeft + chartWidth)
      .attr('y2', (d: any, i: number) => paddingTop + chartHeight / ticks * i + .5)
      .attr('stroke-dasharray', lineStyleType[splitLine.lineStyle.type])
      .style('stroke', splitLine.lineStyle.color)
      .style('stroke-width', splitLine.lineStyle.width)
      .style('visibility', splitLine.show ? 'visible' : 'hidden')

    gysUpdate
      .attr('x1', paddingLeft)
      .attr('y1', (d: any, i: number) => paddingTop + chartHeight / ticks * i + .5)
      .attr('x2', paddingLeft + chartWidth)
      .attr('y2', (d: any, i: number) => paddingTop + chartHeight / ticks * i + .5)
      .attr('stroke-dasharray', lineStyleType[splitLine.lineStyle.type])
      .style('stroke', splitLine.lineStyle.color)
      .style('stroke-width', splitLine.lineStyle.width)
      .style('visibility', splitLine.show ? 'visible' : 'hidden')

    gysExit
      .style('stroke', 'transparent')
      .remove()
  }
}

export default SplitLine
