import { getElem } from '../util/getElem'
import Axis from '../coord/Axis'
import { lineStyleType } from '../config'
import SplitLine from './SplitLine'

interface IxAxis {
  chartWidth: number
  chartHeight: number
  paddingLeft: number
  paddingTop: number
  svg: HTMLDocument
  xAxis: any[]
}

class XAxis {
  splitline: any
  constructor() {
    this.splitline = new SplitLine()
  }

  init(xAxisOption: IxAxis) {
    const {
      chartWidth,
      chartHeight,
      paddingLeft,
      paddingTop,
      xAxis,
      svg,
    } = xAxisOption

    const {
      axisLabel,
      axisLine,
      data,
      splitLine,
      axisTick,
    } = xAxis[0]

    const len = data.length

    const xAxisPrama = {
      orientation: 'bottom',
      ticks: len,
      tickPadding: axisTick.show && 3 || 9,
      tickSize: axisTick.show && 6 || 0,
      tickFormat: (d: any, i: number) => {
        if (i !== len) { // ticks最后一个text不放值
          return data[i]
        }
      },
    }
    const scaleOption = {
      domain: [1, len + 1], // x轴多一份为将值放在中间
      range: [0, chartWidth],
    }

    // 生成坐标轴 gx
    const gx = getElem(svg, 'g', 'yo-xAxis')
    gx.attr('transform', `translate(${paddingLeft}, ${chartHeight + paddingTop})`)
    gx.call(Axis(xAxisPrama, scaleOption))

    // 设置轴文字
    const everyWidth = chartWidth / len
    this.gxTextSet(gx, axisLabel, everyWidth)

    // 设置轴线
    this.gxLineSet(gx, axisLine)

    // 生成 aplitline
    if (JSON.stringify(splitLine) !== '{}') {
      const splitLineOption = {
        splitLine,
        chartWidth,
        chartHeight,
        paddingLeft,
        paddingTop,
        svg,
        len,
      }
      this.splitline.initX(splitLineOption)
    }
  }

  // 设置轴文字 gxText
  gxTextSet(gx: any, axisLabel: any, everyWidth: any) {
    const gxText = gx
      .selectAll('.tick')
      .select('text')
    gxText
      .attr('transform', `translate(${everyWidth / 2}, 0), rotate(${-axisLabel.rotate}, 0 9)`)
      .attr('style', `color: ${axisLabel.color};
        font-family: ${axisLabel.fontFamily};
        font-size: ${axisLabel.fontSize}px`)
      // .attr('text-anchor', axisLabel.rotate && 'end' || 'middle') // 绕文字末置点旋转
      .style('display', axisLabel.show ? 'block' : 'none')
  }

  // 设置轴线 gxLine
  gxLineSet(gx: any, axisLine: any) {
    const gxLine = gx.select('path')
    gxLine
      .style('display', axisLine.show ? 'block' : 'none')
      .attr('stroke-width', axisLine.lineStyle.width)
      .attr('stroke-dasharray', lineStyleType[axisLine.lineStyle.type])
      .attr('stroke', axisLine.lineStyle.color)
  }
}

export default XAxis
