import { getElem } from '../util/getElem'
import Axis from '../coord/Axis'
import { lineStyleType } from '../config'
import SplitLine from './SplitLine'

interface IYAxis {
  chartWidth: number
  chartHeight: number
  paddingLeft: number
  paddingTop: number
  svg: HTMLDocument
  yAxis: any
  series: any
  max: number
  ticks: number
}

class YAxis {
  splitline: any
  constructor() {
    this.splitline = new SplitLine()
  }

  init(yAxisOption: IYAxis) {
    const {
      chartWidth,
      chartHeight,
      paddingLeft,
      paddingTop,
      yAxis,
      svg,
      max,
      ticks,
    } = yAxisOption

    const {
      axisLabel,
      axisLine,
      axisTick,
      inverse,
      name,
      nameGap,
      nameLocation,
      nameRotate,
      nameTextStyle,
      splitLine,
    } = yAxis

    const yAxisPrama = {
      orientation: 'left',
      ticks,
      tickPadding: axisTick.show && 3 || 9,
      tickSize: axisTick.show && 6 || 0,
    }

    const scaleOption = {
      domain: [max, 0],
      range: [0, chartHeight],
    }

    // 坐标轴 gy
    const gy = getElem(svg, 'g', 'yo-yAxis')
    gy.attr('transform', `translate(${paddingLeft}, ${paddingTop})`)
    gy.call(Axis(yAxisPrama, scaleOption))

    // 标签位置
    const location: { [key: string]: any } = {
      start: {
        x: paddingLeft - nameGap,
        y: paddingTop,
      },
      middle: {
        x: paddingLeft - nameGap,
        y: paddingTop + chartHeight / 2,
      },
      end: {
        x: paddingLeft - nameGap,
        y: paddingTop + chartHeight,
      },
    }

    // 坐标轴 name
    const axisName = getElem(svg, 'g', 'yo-xAxis-name')

    const axisText = getElem(axisName, 'text', 'yo-name-text')
    axisText
      .attr('text-anchor', 'middle')
      .style('fill', nameTextStyle.color)
      .style('font-family', nameTextStyle.fontFamily)
      .style('font-size', nameTextStyle.fontSize)
      .text(name)
      .transition()
      .duration(0)
      .attr('transform', `rotate(${-nameRotate}, ${location[nameLocation].x} ${location[nameLocation].y}),
        translate(${location[nameLocation].x}, ${location[nameLocation].y})`)

    // 设置轴文字
    this.gyTextSet(gy, axisLabel)

    // 设置轴线
    this.gyLineSet(gy, axisLine)

    // 生成 aplitline
    if (JSON.stringify(splitLine) !== '{}') {
      const splitLineOption = {
        splitLine,
        chartWidth,
        chartHeight,
        paddingLeft,
        paddingTop,
        svg,
        ticks,
      }
      this.splitline.initY(splitLineOption, 'yAxis')
    }
  }

  // 设置轴文字 gyText
  gyTextSet(gy: any, axisLabel: any) {
    const gyText = gy
      .selectAll('.tick')
      .select('text')
    gyText
      .attr('style', `color:${axisLabel.color};
        font-family: ${axisLabel.fontFamily};
        font-size: ${axisLabel.fontSize}px`)
      .style('display', axisLabel.show ? 'block' : 'none')
  }

  // 设置轴线 gyLine
  gyLineSet(gy: any, axisLine: any) {
    const gyLine = gy.select('path')
    gyLine
      .style('display', axisLine.show ? 'block' : 'none')
      .attr('stroke-width', axisLine.lineStyle.width)
      .attr('stroke-dasharray', lineStyleType[axisLine.lineStyle.type])
      .attr('stroke', axisLine.lineStyle.color)
  }
}

export default YAxis
