import * as d3Aixs from 'd3-axis'
import * as d3Format from 'd3-format'
import * as d3Time from 'd3-time'
import Scale from './Scale'

const d3 = Object.assign(
  {},
  d3Aixs,
  d3Format,
  d3Time,
)

/**
 * return the axis generator
 * @param {Iaxis} axis
 * @param {Iscale} scale
 * @return {Function} Axis
 */
const Axis = (axis?: Iaxis, scale?: Iscale) => {
  const defaultOption = {
    ticks: axis && axis.ticks || 5, // 刻度数
    tickSize: axis && typeof axis.tickSize === 'number'
      ? axis.tickSize : 6, // tickSizeInner = tickSizeOuter = tickSize, tickSize* 存在时失效, 默认值 6
    tickValues: axis && axis.tickValues || null, // 刻度值
    tickFormat: axis && axis.tickFormat || null, // 值格式
    tickPadding: axis && axis.tickPadding || 3, // 文字与刻度之间的间距
    orientation: axis && axis.orientation || 'left', // 轴位置
  }

  let value = null

  switch (defaultOption.orientation) {
    case 'left':
      value = d3.axisLeft(Scale(scale))
      break
    case 'right':
      value = d3.axisRight(Scale(scale))
      break
    case 'top':
      value = d3.axisTop(Scale(scale))
      break
    case 'bottom':
      value = d3.axisBottom(Scale(scale))
      break
    default:
      value = d3.axisLeft(Scale(scale))
  }

  return value.ticks(defaultOption.ticks)
    .tickSize(defaultOption.tickSize)
    .tickValues(defaultOption.tickValues)
    .tickFormat(defaultOption.tickFormat)
    .tickPadding(defaultOption.tickPadding)
}

export default Axis
