/**
 * 图形上文案标签
 * @param {HTMLOrSVGElement} gp // 某标签下生成, 例如 g 标签
 * @param {Array} data
 * @param {Object} label // 样式 json
 * @param {Number} opacity // 可选, 透明度
 * @param {Number} duration // 动画时长
 * @param {Number} exitDuration // 可选, 退出动画时长
 * @param {String} name // 类名
 * @param {Number} x1 // 可选, 起始位置 x
 * @param {Number} y1 // 可选, 起始位置 y
 * @param {Function} x2 // 最终位置 x
 * @param {Function} y2 // 最终位置 y
 */

interface IlabelOption {
  gp: any
  data: any
  label: any
  opacity?: number
  duration: number
  exitDuration?: number
  name: string // 类名
  x1?: number
  y1?: number
  x2: (key: number) => number
  y2: (key: number) => number
}

const label = (labelOption: IlabelOption) => {
  const {
    gp,
    data,
    label,
    opacity = 1,
    duration = 500,
    exitDuration = 200,
    name,
    y1,
    x2,
    y2,
  } = labelOption

  const textUpdate = gp.selectAll(`.${name}`).data(data)
  const textEnter = textUpdate.enter()
  const textExit = textUpdate.exit()

  textEnter.append('text')
    .attr('class', name)
    .attr('x', (d: any, i: number) => x2(i))
    .attr('y', y1)
    .style('fill', label.color)
    .style('font-family', label.fontFamily)
    .style('font-size', `${label.fontSize}px`)
    .style('visibility', label.show && 'visible' || 'hidden')
    .style('opacity', 0)
    .attr('text-anchor', 'middle')
    .text((d: any) => d)
    .transition()
    .duration(duration)
    .style('opacity', opacity)
    .attr('y', (d: any) => y2(d))

  textUpdate
    .transition()
    .duration(duration)
    .attr('x', (d: any, i: number) => x2(i))
    .attr('y', (d: any) => y2(d))
    .style('fill', label.color)
    .style('font-family', label.fontFamily)
    .style('font-size', `${label.fontSize}px`)
    .style('visibility', label.show && 'visible' || 'hidden')
    .style('opacity', opacity)
    .text((d: any) => d)

  textExit
    .transition()
    .duration(exitDuration)
    .style('opacity', 0)
    .remove()
}

export default label
