import * as d3Select from 'd3-selection'
import { getElem } from '../util/getElem'

const d3 = Object.assign(
  {},
  d3Select,
)

interface ItriggerOption {
  chartHeight: number
  paddingLeft: number
  paddingTop: number
  svg: HTMLOrSVGElement
  data: any[]
  rectBoxWidth: number
  tooltip: any
}

const trigger = (triggerOption: ItriggerOption) => {
  const {
    chartHeight,
    paddingLeft,
    paddingTop,
    svg,
    data,
    rectBoxWidth,
    tooltip,
  } = triggerOption
  // gt
  const gt = getElem(svg, 'g', 'yo-trigger')
  const update = gt.selectAll('.yo-trigger-rect').data(data[0].data)
  const enter = update.enter()
  const exit = update.exit()

  enter.append('rect')
    .attr('class', 'yo-trigger-rect')
    .attr('width', rectBoxWidth)
    .attr('height', chartHeight)
    .attr('style', `fill: transparent; opacity: .2; cursor: pointer`)
    .attr('transform', (d: any, i: any) => `translate(${rectBoxWidth * i + paddingLeft}, ${paddingTop})`)
    .on('mouseenter', function(d: any, i: any) {
      d3.select(this)
        .style('fill', '#d2d2d2')
      tooltip.changeIndex(i)
    })
    .on('mouseleave', function() {
      d3.select(this)
        .style('fill', 'transparent')
      tooltip.changeIndex(null)
    })

  update
    .attr('width', rectBoxWidth)
    .attr('height', chartHeight)
    .attr('transform', (d: any, i: any) => `translate(${rectBoxWidth * i + paddingLeft}, ${paddingTop})`)

  exit.remove()
}

export default trigger
