import React from 'react'
import echarts from 'echarts'
import {
  getTooltipOption,
  getLegendOption,
  getXAxisOption,
  getYAxisOption,
  getSeriesOption,
} from '../generator'

interface IProps {
  options: Ioptions
}

const generator = (option: Ioptions) => {
  const { chartStyles, data, cols, metrics } = option
  const { xAxis, yAxis, legend, splitLine, label, spec } = chartStyles

  const legendOption = {
    type: 'bar',
    legend,
    data,
    cols,
    metrics,
  }

  const xAxisOption = {
    xAxis,
    data,
    cols,
    splitLine,
    mainAxisOption: { mainAxis: true },
  }

  const yAxisOption = {
    yAxis,
    data,
    splitLine,
    cols,
    metrics,
  }

  const seriesOption = {
    type: 'stackbar',
    data,
    label,
    cols,
    spec,
    metrics,
  }

  return {
    tooltip: getTooltipOption('shadow'),
    legend: getLegendOption(legendOption),
    xAxis: getXAxisOption(xAxisOption),
    yAxis: getYAxisOption(yAxisOption),
    series: getSeriesOption(seriesOption),
  }
}

let chart: any
const initChart = (type: string, id: HTMLDivElement, option: Ioptions) => {
  if (type === 'init') {
    chart = echarts.init(id)
  }
  chart.setOption(generator(option))
}

class Chart extends React.Component<IProps, any> {
  div: HTMLDivElement

  componentDidMount() {
    const { options } = this.props
    initChart('init', this.div, options)
  }

  componentDidUpdate() {
    const { options } = this.props
    initChart('update', this.div, options)
  }

  render() {
    const { width, height } = this.props.options
    return (
      <div style={{ width, height }} ref={(node) => this.div = node} />
    )
  }
}

export default Chart
