/**
 * legend option
 * @param {Ilegend} legend
 * @param {Array} data
 * @return {Object}
 */

import { colsValue, legendSelected, dataKey } from './util'
import { defaultFontSize } from '../../util/defalut.config'
import { chartType } from '../../util/config'

interface IlegendOption {
  type: string
  legend: Ilegend
  data: any[]
  cols: any[]
  metrics?: any[]
}

const getLegendOption = (legendOption: IlegendOption) => {
  const {
    type,
    legend,
    data,
    cols,
    metrics,
  } = legendOption

  const {
    color,
    fontFamily,
    fontSize,
    legendPosition,
    selectAll,
    showLegend,
    orient,
  } = legend

  let position

  switch (legendPosition) {
    case 'right':
      position = { top: 0, right: '8%' }
      break
    case 'left':
      position = { top: 0, left: '8%' }
      break
    case 'top':
      position = { bottom: 0, left: '10%' }
      break
    case 'bottom':
      position = { bottom: 0, right: '10%' }
      break
    default:
      position = { right: '10%' }
  }

  let dataSelected
  if (type === 'pie') {
    dataSelected = {
      data: colsValue(data, cols),
      selected: legendSelected(type, selectAll, data, cols),
      orient: orient || 'vertical',
    }
  }
  if (chartType.includes(type)) {
    dataSelected = {
      data: dataKey(data, cols, metrics),
      selected: legendSelected(type, selectAll, data, cols, metrics),
      orient: orient || 'horizontal',
    }
  }

  return {
    show: showLegend,
    textStyle: {
      fontFamily,
      fontSize: fontSize || defaultFontSize,
      color,
    },
    ...position,
    ...dataSelected,
    selectedMode: 'multiple',
  }
}

export default getLegendOption
