/**
 * radar(雷达图) option
 * @param {Array} data
 * @param {Array} cols
 * @return {Object}
 */

import { colsValue, dataKey, dataValue } from './util'

interface IradarOption {
  data: any[]
  cols: any[]
  spec: Ispec
  metrics: any[]
}

const getRadarOption = (radarOption: IradarOption) => {
  const {
    data,
    cols,
    spec,
    metrics,
  } = radarOption

  const { radarShape } = spec
  const colsvalue = colsValue(data, cols)
  const datakey = dataKey(data, cols, metrics)
  const datavalue = dataValue(data, cols, metrics)

  // 取得每条数据中的最大值
  const maxArr = datakey.map((item: any) => Math.max(...datavalue[item]))

  // 取得最终最大值
  const max = Math.max(...maxArr)

  return {
    shape: radarShape || 'radar',
    indicator: colsvalue.map((item) => {
      return {
        name: item,
        max,
      }
    }),
    splitArea: {
      show: false,
    },
  //   name: {
  //     textStyle: {
  //       color: 'rgb(238, 197, 102)',
  //     },
  //   },
  //   axisLine: {
  //     lineStyle: {
  //       color: 'rgba(238, 197, 102, 0.5)',
  //     },
  //   },
  //   splitLine: {
  //   lineStyle: {
  //     color: [
  //       'rgba(238, 197, 102, 0.6)',
  //     ].reverse(),
  //   },
  // },
  }
}

export default getRadarOption
