/**
 * series option
 * @param {Array} data
 * @return {Array}
 */

import { defaultFontSize } from '../../util/defalut.config'
import { chartType } from '../../util/config'
import { colsValue, dataKey, dataValue } from './util'

interface IseriesOption {
  type: string
  data: any[]
  label: Ilabel
  cols: any[]
  spec?: Ispec
  mixedlinebar?: Imixedlinebar
  metrics?: any[]
}

const getSeriesOption = (seriesOption: IseriesOption) => {

  const {
    type,
    data,
    label,
    cols,
    spec,
    mixedlinebar,
    metrics,
  } = seriesOption

  const {
    labelColor,
    labelFontFamily,
    labelFontSize,
    labelPosition,
    showLabel,
    pieLabelPosition,
  } = label

  const {
    roseType,
    circle,
    smooth,
    step,
  } = spec

  if (chartType.includes(type)) {
    const smoothOption = spec.hasOwnProperty('smooth') ? { smooth } : null
    const stepOption = spec.hasOwnProperty('step') ? { step } : null

    return dataKey(data, cols, metrics).map((item: any) => {
      return {
        name: item,
        type,
        data: dataValue(data, cols, metrics)[item],
        label: {
          show: showLabel,
          color: labelColor,
          fontSize: labelFontSize || defaultFontSize,
          fontFamily: labelFontFamily,
          position: labelPosition,
        },
        ...smoothOption,
        ...stepOption,
      }
    })
  }

  if (type === 'stackbar') {
    return dataKey(data, cols, metrics).map((item: any) => {
      return {
        name: item,
        type: 'bar',
        stack: '总量',
        data: dataValue(data, cols, metrics)[item],
        label: {
          show: showLabel,
          color: labelColor,
          fontSize: labelFontSize || defaultFontSize,
          fontFamily: labelFontFamily,
          position: labelPosition,
        },
      }
    })
  }

  if (type === 'polarbar') {
    return dataKey(data, cols, metrics).map((item: any) => {
      return {
        name: item,
        type: 'bar',
        stack: 'polar',
        data: dataValue(data, cols, metrics)[item],
        coordinateSystem: 'polar',
        label: {
          show: showLabel,
          color: labelColor,
          fontSize: labelFontSize || defaultFontSize,
          fontFamily: labelFontFamily,
          position: labelPosition,
        },
      }
    })
  }

  if (type === 'linebar') {
    const smoothOption = spec.hasOwnProperty('smooth') ? { smooth } : null
    const stepOption = spec.hasOwnProperty('step') ? { step } : null

    return dataKey(data, cols, metrics).map((item: any, index: number) => {
      return {
        name: item,
        type: mixedlinebar[item] || 'bar',
        // yAxisIndex: index,
        data: dataValue(data, cols, metrics)[item],
        label: {
          show: showLabel,
          color: labelColor,
          fontSize: labelFontSize || defaultFontSize,
          fontFamily: labelFontFamily,
          position: labelPosition,
        },
        ...smoothOption,
        ...stepOption,
      }
    })
  }

  if (type === 'pie') {
    const datakey = dataKey(data, cols, metrics)[0] // 一个指标
    const datavalue = dataValue(data, cols, metrics)
    const colsvalue = colsValue(data, cols)
    const name = colsvalue.map((item, index) => {
      return {
        name: item,
        value: datavalue[datakey][index],
      }
    })

    return [{
      type: 'pie',
      radius : circle ? ['50%', '70%'] : '70%', // 环形图
      data: name,
      roseType, // 南丁格儿图
      itemStyle: {
        emphasis: {
          shadowBlur: 10,
          shadowOffsetX: 0,
          shadowColor: 'rgba(0, 0, 0, 0.5)',
        },
      },
      label: {
        show: showLabel,
        position: pieLabelPosition,
        color: labelColor,
        fontFamily: labelFontFamily,
        fontSize: labelFontSize || defaultFontSize,
      },
    }]
  }

  if (type === 'funnel') {
    const datakey = dataKey(data, cols, metrics)[0] // 一个指标
    const datavalue = dataValue(data, cols, metrics)
    const colsvalue = colsValue(data, cols)
    const name = colsvalue.map((item, index) => {
      return {
        name: item,
        value: datavalue[datakey][index],
      }
    })

    return {
      type,
      data: name,
      left: '10%',
      width: '80%',
      minSize: '0%',
      maxSize: '90%',
      sort: 'descending',
      gap: 2,
      itemStyle: {
        emphasis: {
          shadowBlur: 10,
          shadowOffsetX: 0,
          shadowColor: 'rgba(0, 0, 0, 0.5)',
        },
      },
      label: {
        show: showLabel,
        position: pieLabelPosition,
        color: labelColor,
        fontFamily: labelFontFamily,
        fontSize: labelFontSize || defaultFontSize,
      },
    }
  }

  if (type === 'radar') {
    return {
      name: type,
      type,
      data: dataKey(data, cols, metrics).map((item: any) => {
        return {
          name: item,
          value: dataValue(data, cols, metrics)[item],
        }
      }),
      symbol: 'none',
      // itemStyle: {
      //   normal: {
      //     color: 'rgb(238, 197, 102)'
      //   },
      // },
      areaStyle: {
        normal: {
          opacity: 0.05,
        },
      },
    }
  }
}

export default getSeriesOption
