/**
 * xAxis option
 * @param {Array} data
 * @return {Object}
 */

import { defaultFontSize } from '../../util/defalut.config'
import { colsValue } from './util'

interface IXAxisOption {
  xAxis: IxAxis
  data: any[]
  cols: any[]
  splitLine: IsplitLine
  mainAxisOption?: any
}
const getXAxisOption = (xAxisOption: IXAxisOption) => {
  const {
    xAxis,
    data,
    cols,
    splitLine,
    mainAxisOption,
  } = xAxisOption

  const {
    showLabel,
    labelColor,
    labelFontFamily,
    labelFontSize,
    showLine,
    lineColor,
    lineSize,
    lineStyle,
    xAxisInterval,
    xAxisRotate,
    showTicks,
  } = xAxis

  const {
    showVerticalLine,
    verticalLineColor,
    verticalLineSize,
    verticalLineStyle,
  } = splitLine

  const mainAxis = mainAxisOption && mainAxisOption.mainAxis

  const dataOption = {
    data: mainAxis ? colsValue(data, cols) : null,
  }

  return [{
    ...dataOption,

    // x轴 线
    axisLine: {
      show: showLine,
      lineStyle: {
        color: lineColor,
        width: +lineSize,
        type: lineStyle,
      },
    },

    // x轴 标签
    axisLabel: {
      show: showLabel,
      color: labelColor,
      fontFamily: labelFontFamily,
      fontSize: +labelFontSize || defaultFontSize,
      interval: xAxisInterval,
      rotate: xAxisRotate,
    },

    // 垂直分割线
    splitLine: {
      show: showVerticalLine,
      lineStyle: {
        color: verticalLineColor,
        width: +verticalLineSize,
        type: verticalLineStyle,
      },
    },

    // 刻度
    axisTick: {
      show: showTicks,
    },
  }]
}

export default getXAxisOption
