/**
 * yAxis option
 * @param {IyAxis} yAxis
 * @return {Object}
 */

import { defaultFontSize } from '../../util/defalut.config'
import { colsValue } from './util'

interface IYAxisOption {
  yAxis: IyAxis
  data: any[]
  splitLine: IsplitLine
  cols: any[]
  mainAxisOption?: any
  metrics?: any[]
}

const getYAxisOption = (yAxisOption: IYAxisOption) => {
  const {
    yAxis,
    data,
    splitLine,
    cols,
    mainAxisOption,
    metrics,
  } = yAxisOption

  const {
    showLabel,
    labelColor,
    labelFontFamily,
    labelFontSize,
    showLine,
    lineColor,
    lineSize,
    lineStyle,
    showTitleAndUnit,
    nameGap,
    nameLocation,
    nameRotate,
    titleColor,
    titleFontFamily,
    titleFontSize,
    inverse,
    showTicks,
  } = yAxis

  const {
    showHorizontalLine,
    horizontalLineColor,
    horizontalLineSize,
    horizontalLineStyle,
  } = splitLine

  const mainAxis = mainAxisOption && mainAxisOption.mainAxis

  // 取名称
  const name: any = []
  if (data && data.length > 0) {
    const { ...value } = data[0]
    for (const key in value) {
      if (key !== cols[0]) {
        name.push(key.match(/\(.+?\)/g)[0].slice(1, -1))
      }
    }
  }

  // 替换别名
  if (metrics) {
    metrics.map((item) => {
      const currentName = item.name.split('@')[0]
      if (name.indexOf(currentName) > -1) {
        name[name.indexOf(currentName)] = item.alias || currentName
      }
    })
  }

  const dataOption = {
    data: mainAxis ? colsValue(data, cols) : null,
  }

  return {
    ...dataOption,

    // y轴 标题和单位
    name: showTitleAndUnit && name.join('/') || '',
    nameGap,
    nameLocation,
    nameRotate,
    inverse,
    nameTextStyle: {
      color: titleColor,
      fontFamily: titleFontFamily,
      fontSize: +titleFontSize || defaultFontSize,
    },

    // y轴 线
    axisLine: {
      show: showLine,
      lineStyle: {
        color: lineColor,
        width: +lineSize,
        type: lineStyle,
      },
    },

    // y轴 标签
    axisLabel: {
      show: showLabel,
      color: labelColor,
      fontFamily: labelFontFamily,
      fontSize: +labelFontSize || defaultFontSize,
    },

    // 水平分割线
    splitLine: {
      show: showHorizontalLine,
      lineStyle: {
        color: horizontalLineColor,
        width: +horizontalLineSize,
        type: horizontalLineStyle,
      },
    },

    // 刻度
    axisTick: {
      show: showTicks,
    },
  }
}

export default getYAxisOption
