import { chartType } from '../../util/config'

/**
 * get legend | series key
 * @param {Array} data
 * @return {Array}
 */
const dataKey = (data: any[], cols: any[], metrics?: any[]) => {
  const name: any = []
  if (data && data.length > 0) {
    const { ...value } = data[0]
    for (const key in value) {
      if (key !== cols[0]) {
        name.push(key.match(/\(.+?\)/g)[0].slice(1, -1))
      }
    }
  }

  // 替换别名
  if (metrics) {
    metrics.map((item) => {
      const currentName = item.name.split('@')[0]
      if (name.indexOf(currentName) > -1) {
        name[name.indexOf(currentName)] = item.alias || currentName
      }
    })
  }

  return name
}

/**
 * get data array value
 * @param {Array} data
 * @return {Object}
 */
const dataValue = (data: any[], cols: any[], metrics?: any[]) => {
  if (data && data.length > 0) {
    const { ...value } = data[0]
    const obj: any = {}
    for (const key in value) {
      if (key !== cols[0]) {
        let name = key.match(/\(.+?\)/g)[0].slice(1, -1)

        // 替换别名
        if (metrics) {
          metrics.map((item) => {
            const currentName = item.name.split('@')[0]
            if (name === currentName) {
              name = item.alias || currentName
            }
          })
        }

        obj[name] = data.map((item) => item[key])
      }
    }

    return obj
  }
}

/**
 * get cols array value
 * @param {Array} data
 * @param {Array} cols
 * @return {Array}
 */
const colsValue = (data: any[], cols: any[]) => {
  if (data && data.length > 0) {
    return data.map((item) => item[cols[0]])
  }
  return []
}

/**
 * legend select
 * @param {boolean} selectAll
 * @param {Array} data
 * @return {Object}
 */
const legendSelected = (type: string, selectAll: boolean, data: any[], cols: any[], metrics?: any[]) => {
  const obj: any = {}
  if (chartType.includes(type)) {
    dataKey(data, cols, metrics).map((item: any) => {
      obj[item] = selectAll ? true : false
    })
  }

  if (type === 'pie') {
    colsValue(data, cols).map((item) => {
      obj[item] = selectAll ? true : false
    })
  }

  return obj
}

/**
 * tooltip option
 * @param {String} 'shadow' | 'cross'
 * @return {Object}
 */
const getTooltipOption = (type: string) => {
  return {
    show: true,
    trigger: type === 'item' ? 'item' : 'axis',
    axisPointer: {
      type,
    },
  }
}

export {
  dataKey,
  dataValue,
  colsValue,
  legendSelected,
  getTooltipOption,
}
